@@ -7,7 +7,7 @@ from contextlib import contextmanager
 from io import StringIO
 from collections import namedtuple, defaultdict
 from pathlib import Path
-from psutil import cpu_times
+import psutil
 from typing import Dict, Tuple, Optional
 
 from pytimings.mpi import get_communication_wrapper, get_local_communicator, get_communicator
@@ -16,29 +16,30 @@ from pytimings.tools import ensure_directory_exists
 try:
     PERF_COUNTER_FUNCTION = time.perf_counter_ns
     THREAD_TIME_FUNCTION = time.time_ns
-    TO_SECONDS_FACTOR = 1e-6
+    TO_SECONDS_FACTOR = 1e-9
 except AttributeError:
-    PERF_COUNTER = time.perf_counter
+    PERF_COUNTER_FUNCTION = time.perf_counter
     try:
         THREAD_TIME_FUNCTION = time.thread_time
     except AttributeError:
         # TODO Log warning
         THREAD_TIME_FUNCTION = time.process_time
-    TO_SECONDS_FACTOR = 1e-3
+    TO_SECONDS_FACTOR = 1
 
 THREAD_TIME = "thread"
 WALL_TIME = "wall"
 SYS_TIME = "sys"
 USER_TIME = "user"
-IDLE_TIME = "idle"
 
-TimingDelta = namedtuple(
-    "TimingDelta", [WALL_TIME, SYS_TIME, USER_TIME], defaults=[0, 0, 0]
-)
+TimingDelta = namedtuple("TimingDelta", [WALL_TIME, SYS_TIME, USER_TIME], defaults=[0, 0, 0])
 
 
-class NoTimerError(RuntimeError):
-    pass
+class NoTimerError(Exception):
+    def __init__(self, section, timings=None):
+        self.section = section
+        self.timings = timings or global_timings
+        avail = 'Available sections: ' + ','.join(self.timings._known_timers_map.keys())
+        super().__init__(f"trying to access timer for unknown section '{section}'\n{avail}")
 
 
 class TimingData:
@@ -46,14 +47,14 @@ class TimingData:
         self.name = name
         self._end_resources = None
         self._end_times = None
+        self._process = psutil.Process()
         self._start_times = self._get()
 
     def _get(self):
-        ps_times = cpu_times()
+        ps_times = self._process.cpu_times()
         return {
             USER_TIME: ps_times.user,
             SYS_TIME: ps_times.system,
-            IDLE_TIME: ps_times.idle,
             WALL_TIME: PERF_COUNTER_FUNCTION(),
             THREAD_TIME: THREAD_TIME_FUNCTION(),
         }
@@ -64,9 +65,7 @@ class TimingData:
     def delta(self):
         delta_times = self._end_times or self._get()
 
-        wall = (
-            delta_times[WALL_TIME] - self._start_times[WALL_TIME]
-        ) * TO_SECONDS_FACTOR
+        wall = (delta_times[WALL_TIME] - self._start_times[WALL_TIME]) * TO_SECONDS_FACTOR
         # kernel resource usage already is in seconds
         return TimingDelta(
             wall,
@@ -75,16 +74,16 @@ class TimingData:
         )
 
 
+def _default_timer_dict_entry():
+    return (False, None)
+
+
 class Timings:
     def __init__(self):
         self._commited_deltas: Dict[str, TimingDelta] = {}
-        self._output_dir = None
-        self._known_timers_map: Dict[
-            str, Tuple[bool, Optional[TimingData]]
-        ] = defaultdict(lambda _: (False, None))
+        self._known_timers_map: Dict[str, Tuple[bool, Optional[TimingData]]] = defaultdict(_default_timer_dict_entry)
         self._csv_sep = ","
         self.reset()
-        self.set_outputdir("profiling")
 
     def start(self, section_name: str) -> None:
         """set this to begin a named section"""
@@ -102,9 +101,7 @@ class Timings:
                 self.stop(section)
             return
         if section_name not in self._known_timers_map.keys():
-            raise NoTimerError(
-                f"Trying to stop section {section_name} for which no timer was running"
-            )
+            raise NoTimerError(section_name, self)
         self._known_timers_map[section_name] = (
             False,
             self._known_timers_map[section_name][1],
@@ -116,10 +113,9 @@ class Timings:
             self._commited_deltas[section_name] = delta
         else:
             previous_delta = self._commited_deltas[section_name]
-            new_delta = TimingDelta(*tuple(
-                map(sum, zip(delta, previous_delta)))
-            )
+            new_delta = TimingDelta(*tuple(map(sum, zip(delta, previous_delta))))
             self._commited_deltas[section_name] = new_delta
+        return self._commited_deltas[section_name]
 
     def reset(self, section_name: str = None) -> None:
         """set elapsed time back to 0 for a given section or all of them if section_name is None"""
@@ -139,9 +135,12 @@ class Timings:
 
     def delta(self, section_name: str) -> Dict[str, int]:
         """get the full delta dict"""
-        return self._commited_deltas[section_name]
+        try:
+            return self._commited_deltas[section_name]
+        except KeyError:
+            raise NoTimerError(section_name, self)
 
-    def output_per_rank(self, csv_base: str) -> None:
+    def output_per_rank(self, output_dir: Path, csv_base: str) -> None:
         """
              creates one file local to each MPI-rank (no global averaging)
         *  one single rank-0 file with all combined/averaged measures
@@ -149,7 +148,8 @@ class Timings:
         communication = get_communication_wrapper()
         rank = communication.rank
 
-        filename = self._output_dir / f"{csv_base}_p{rank:08d}.csv"
+        ensure_directory_exists(output_dir)
+        filename = output_dir / f"{csv_base}_p{rank:08d}.csv"
         with open(filename, "wt") as outfile:
             self.output_all_measures(outfile, get_local_communicator())
         tmp_out = StringIO()
@@ -157,8 +157,10 @@ class Timings:
         self.output_all_measures(tmp_out, get_communicator())
         # but only rank 0 needs to write it
         if rank == 0:
-            a_filename = dir / f"{csv_base}.csv"
-            open(a_filename, "wt").write(tmp_out)
+            a_filename = output_dir / f"{csv_base}.csv"
+            tmp_out.seek(0)
+            open(a_filename, "wt").write(tmp_out.read())
+            return a_filename
 
     def output_simple(self, out=None):
         """outputs walltime only w/o MPI-rank averaging"""
@@ -207,10 +209,6 @@ class Timings:
         if comm.rank == 0:
             shutil.copyfileobj(stash, out)
 
-    def set_outputdir(self, dirname: str) -> None:
-        self._output_dir = Path(dirname).resolve()
-        ensure_directory_exists(self._output_dir)
-
 
 global_timings = Timings()
 
@@ -222,8 +220,6 @@ def scoped_timing(section_name, log_function=None, timings=None):
     try:
         yield
     finally:
-        duration = timings.stop(section_name)
+        delta = timings.stop(section_name)
         if log_function:
-            log_function(
-                f"Executing{section_name} took {duration * TO_SECONDS_FACTOR}s\n"
-            )
+            log_function(f"Executing {section_name} took {delta.wall * TO_SECONDS_FACTOR}s\n")
